#' @title impute.matrix.numerical
#' @aliases impute.matrix.numerical
#' @description Imputation method for numerical variables. Optimized code 
#' for regression prediction
#' @param x A matrix with numerical variables with missing values
#' @param rule possible values are \code{"fixed"} and \code{"random"}
#' @param max.iter maximum numbers of iterations
#' @param immediate.subst imputed column is immediately used as a predictor (but \code{cov(x)} is not updated until the iteration is completed
#' @param max.err max mean relative error (evaluated only for numerical variables between two successive iterations)
#' @param order.var logical. If \code{order.var=TRUE} variables are reordered. (yet not implemented)
#' @param reduced.rows logical. If \code{order.var=TRUE} then a optimized sample of rows is taken 
#' to compute variace covariance matrices. (yet not implemented)
#' @details \code{impute.num.simple}  is used for numerical variables at the first step. 
#'  Imputation of numerical variables is iterated until convergence (or until max.iter iterations are completed). Imputation uses multiple regression:
#'  no use of \code{lm} is made.
#  In future a version a random assignment of missing values,
#' conditioned to other variables, will be implemented, with \code{rule}=\code{"random"}
#' @family  imputations impute.matrix.mixed impute.matrix.numerical impute.num.simple impute.cat.simple
#' @seealso  \code{\link{impute.matrix.mixed}} 
#' \code{\link{impute.matrix.numerical}} 
#' \code{\link{impute.cat.simple}} \code{\link{impute.num.simple}}
#' @return the matrix \code{x} with imputed missing values. 
impute.matrix.numerical<-function(x, rule="fixed",max.iter=5,immediate.subst=TRUE, order.var=FALSE, reduced.rows=FALSE,max.err=0.01){
  n     =nrow(x)
  k     =ncol(x)
  mat.miss=is.na(x)

#  
#  option order.var not used 
#
  num.miss.vars=colMeans(mat.miss)
  ind.vars=order(num.miss.vars,decreasing = FALSE)
  ind.inv.vars=order(ind.vars)
  if (order.var) x=x[,ind.vars]
  ##
  ## first round of imputation
  ##
  for (j in 1:k){
    z=x[,j]
    z=  impute.num.simple(z,rule="mean.subst")
    x[,j]=z
  }
  iter=0
  delta=sum(abs(x))
  ##
  ## cycle of max.iter imputation (only for numerical variables)
  ##
  while(iter<max.iter&delta>max.err){
    iter=iter+1
    mx=apply(x,2,mean)
    covx=cov(x)
    inv.covx=chol2inv(chol(covx))
    cat("iter ",iter)
    print("")
    delta=0
    xi=x
    for (j in 1:k){
      ind.miss=mat.miss[,j]
      if(sum(as.numeric(ind.miss))>0){
      z=x[,j]
      ind=1:k
      ind=ind[-j]
      b=crossprod(reduced.inv(covx,inv.covx,j),covx[ind,j])
       z[ind.miss]=as.vector(as.matrix(x[ind.miss,ind])%*%b)+mx[j]-sum(b*mx[-j])
       xi[,j]=z
       
       delta=delta+sum(abs(x[,j]-xi[,j]))/sum(abs(x[,j]-mean(abs(x[,j]))))
      if(immediate.subst) x[,j]=xi[,j]
      }
      
    }
    x=xi
    delta=delta/k
    print(delta)
  }
  if (order.var) x=x[,ind.inv.vars]
  
  return(x)}
