#' @title Simple  regression simulation
#' @description Visualization of a simulation of a simple linear regression y~x. The values of x can be chosen. 
#' @param nsample number of samples (and lines) to simulate
#' @param tpause second of pause between samples
#' @param n single sample size
#' @param b0 true value of intercept
#' @param b1 true value of regression coefficient for \code{x}
#' @param x values of x
#' @param color.predicted custom color for predicted points
#' @param color.truepoints custom color for true points
#' @param color.observed custom color for observed (simulated) points
#' @param color.truelines custom color for true line
#' @param single.predictedpoints if the predicted points must be plotted
#' @param single.predictedline if the predicted line must be plotted
#' @param single.true if the true line must be plotted
#' @param winbuffer value for options(windowsBuffered)
#' @return a list with simulated estimates and the x array used 
#' @export
#'
#' @examples
simulreg2d=function(nsample=1,tpause=0.05,	n=10,b0=1,b1=0.8,x=seq(1,10,length.out = n),
                        color.predicted=2,
                        color.truepoints=4,
                        color.observed=1,
                        color.truelines=5,
                        winbuffer=(tpause==0),
                        single.predictedpoints=FALSE,
                        single.predictedline  =TRUE,
                        single.true           =TRUE
                        
){
  nsample=trunc(nsample)
  options(windowsBuffered=winbuffer)
  ey=b0+b1*x
  #par(mfrow=c(1,2))
  xrange=enlarged.range(x,eps=0.2)
  yrange=enlarged.range(ey,eps=0.4)
  
  
  close.screen(all.screens=TRUE)
  split.screen(c(1,2),erase = TRUE)
  
  # true line
  screen(n=1)
  
  plot(x,ey,xlim=xrange,ylim=yrange,main=paste(nsample, "samples of size ",n, "; True parameters ",round(b0,2),round(b1,2)))
  
  abline(a=b0,b=b1,xlim=xrange, col=color.truelines,lwd=3)
  points(x,ey,col=color.truepoints,lwd=3)
  
  screen(2)
  
  plot(x,ey,type="l",xlim=xrange,ylim=yrange,main=paste(" True parameters ",round(b0,2),round(b1,2)))
  
  abline(a=b0,b=b1,xlim=xrange, col=color.truelines,lwd=3)
  points(x,ey,col=color.truepoints,lwd=3)
  
  if(nsample<1)return("Wrong number of samples")
  
  b.simulated=numeric(0)
  
  for (i in 1:nsample){
    y=ey+rnorm(n)
    l1=lm(y~x)
    
    ## left screen: single sample
    ## 
    screen(n=1,new=TRUE)
#    erase.screen(n=1)
    plot(x,y,col=color.observed,xlim=xrange,ylim=yrange, main=paste("sample ",i, "of size ",n))
    #screen(n=1)
    if (single.predictedline){with(l1,abline(a=coefficients[1],b=coefficients[2],xlim=xrange,ylim=yrange))	
      if(single.predictedpoints) points(x,predict(l1),col=color.predicted)
    }
    if(single.true){
      abline(a=b0,b=b1,xlim=xrange, col=color.truelines,lwd=2)
      points(x,ey,col=color.truepoints,lwd=2)
    }
    
    #	screen(n=1,new=TRUE)
    
    #	plot(x,ey,type="l",xlim=xrange,ylim=yrange,main=paste( "sample ",i, "of size ",n, "; True parameters ",round(b0,2),round(b1,2)))
    #	screen(n=1)
    #	with(l1,abline(a=coefficients[1],b=coefficients[2],xlim=xrange,ylim=yrange))
    #	erase.screen(n=1)
    #	erase.screen(n=1)
    #	screen(n=1)
    
    #	points(x,y,col=color.observed)
    
    screen(n=2,new=FALSE)
    plot(x,predict(l1),col=color.predicted,xlim=xrange,ylim=yrange)
    screen(n=2,new=FALSE)
    
    with(l1,abline(a=coefficients[1],b=coefficients[2],xlim=xrange,ylim=yrange))
    b.simulated=rbind(b.simulated,coef(l1))
    
    Sys.sleep(tpause)
    
  }
  screen(n=2,new=FALSE)
  abline(a=b0,b=b1,xlim=xrange, col=color.truelines,lwd=3)
  points(x,ey,col=color.truepoints,lwd=3)
  return(list(matb=b.simulated,matx=x))
  
}
